<?php

namespace Drupal\views_block_with_lazy_builder\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\views\Views;

/**
 * Provides an example block.
 *
 * @Block(
 *   id = "views_bloc_with_lazy_builder_example",
 *   admin_label = @Translation("Views Block with lazy builder"),
 *   category = @Translation("Views Block with lazy builder"),
 *   deriver = "Drupal\views_block_with_lazy_builder\Plugin\Derivative\ViewsBlockWithLazyBuilder"
 * )
 */
class ViewsBlockWithLazyBuilder extends BlockBase implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public function build() {

    $build = [];

    $explodePluginId = explode(':', $this->getPluginId());
    [$viewID, $displayId] = explode('-', $explodePluginId[1]);

    $build['lazy_builder_time'] = [
      '#lazy_builder' => [
        static::class . '::viewsBlockWithLazyBuilderCallback', [
          'viewsId' => $viewID,
          'displayId' => $displayId,
        ],
      ],
      '#create_placeholder' => TRUE,
    ];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return [
      'viewsBlockWithLazyBuilderCallback',
    ];
  }

  /**
   * Lazy Builder which load views block and serve it.
   *
   * @param string $viewId
   *   Views Id.
   * @param string $displayId
   *   Views Display.
   *
   * @return array
   *   Render array.
   */
  public static function viewsBlockWithLazyBuilderCallback($viewId, $displayId) {

    // Get views block.
    $view = Views::getView($viewId);
    $build = [];

    if (is_object($view)) {
      $view->setDisplay($displayId);
      $view->execute();

      // Render the view.
      $build = $view->render();
    }

    return $build;
  }

}
